@@ -31,6 +31,8 @@ our $NA_VALUE = 'NA';
 has [qw/assay col_data row_data row_names col_names row_data_names col_data_names meta_data/] =>
   ( is => 'rw', default => sub { [] } );
 
+has na_value => ( is => 'rw', default => sub { $NA_VALUE } );
+
 sub data { shift->assay(@_) }
 
 sub header { shift->col_names(@_) }
@@ -49,8 +51,10 @@ sub spew_assay {
   my $src   = shift;
   my $param = shift // {};
   my %c     = %$param;
-  $c{header} = $self->col_names if ( $param->{header} || $param->{col_names} );
+  $c{header}    = $self->col_names if ( $param->{header} || $param->{col_names} );
   $c{row_names} = $self->row_names if ( $param->{row_names} );
+  $c{col_data}  = $self->col_data  if ( $param->{col_data} );
+  $c{na_value}  = $self->na_value;
   mspew( $src, $self->assay, \%c );
   return $self;
 }
@@ -123,6 +127,7 @@ sub transpose {
 
   return __PACKAGE__->new(
     assay          => \@assay_t,
+    na_value       => $self->na_value,
     row_names      => Clone::clone( $self->col_names ),
     row_data       => \@row_data_t,
     col_names      => Clone::clone( $self->row_names ),
@@ -173,6 +178,7 @@ sub add_col {
 
 sub cbind {
   my ( $self, $data, $names, $col_data_n ) = @_;
+  my $na_value = $self->na_value;
 
   $col_data_n //= [];
   push @{ $self->col_names }, @$names if ( $names && @$names );
@@ -201,8 +207,8 @@ sub cbind {
 
   my $col_data_ncol = @$col_data > @$col_data_n ? @$col_data : @$col_data_n;
   for ( my $i = 0; $i < $col_data_ncol; $i++ ) {
-    $col_data->[$i] //= [ ($NA_VALUE) x $ncol ];
-    push @{ $col_data->[$i] }, @{ $col_data_n->[$i] // [ ($NA_VALUE) x $ncol_added ] };
+    $col_data->[$i] //= [ (undef) x $ncol ];
+    push @{ $col_data->[$i] }, @{ $col_data_n->[$i] // [ (undef) x $ncol_added ] };
   }
 
   return $self;
@@ -277,6 +283,7 @@ sub subset {
 
   return __PACKAGE__->new(
     assay          => \@assay_new,
+    na_value       => $self->na_value,
     row_names      => \@row_names_new,
     row_data       => \@row_data_new,
     row_data_names => Clone::clone( $self->row_data_names ),
@@ -339,7 +346,7 @@ sub merge {
       next if ( $param{join} eq 'left' && !$data_x );
       $data_y //= {
         idcs      => [-1],
-        rows      => [ [ ($NA_VALUE) x $se_y->ncol ] ],
+        rows      => [ [ (undef) x $se_y->ncol ] ],
         key       => $data_x->{key},
         key_names => $data_x->{key_names},
         row_names => [],
@@ -348,7 +355,7 @@ sub merge {
     }
     if ( $param{join} eq 'right' || $param{join} eq 'full' ) {
       next if ( $param{join} eq 'right' && !$data_y );
-      my @row = ( ($NA_VALUE) x $se_x->ncol );
+      my @row = ( (undef) x $se_x->ncol );
       @row[@$idcs_x] = @{ $data_y->{key} };
       $data_x //= {
         idcs      => [-1],
@@ -377,15 +384,17 @@ sub merge {
   my @col_data;
 
   if ( @$col_data_x || @$col_data_y ) {
-    for ( my $i = 0; $i < $ncol_total; $i++ ) {
-      my $cd_x = $col_data_x->[$i] // [ ($NA_VALUE) x $ncol_only_x ];
-      my $cd_y = $col_data_y->[$i] // [ ($NA_VALUE) x $ncol_only_y ];
+    my $col_data_nrow = max( ( scalar @$col_data_x ), ( scalar @$col_data_y ) );
+    for ( my $i = 0; $i < $col_data_nrow; $i++ ) {
+      my $cd_x = $col_data_x->[$i] // [ (undef) x $ncol_only_x ];
+      my $cd_y = $col_data_y->[$i] // [ (undef) x $ncol_only_y ];
       push @col_data, [ @$cd_x, @{$cd_y}[@$inv_by_y] ];
     }
   }
   return __PACKAGE__->new(
     assay => \@assay_new,
 
+    na_value => $se_x->na_value,
     row_data => \@row_data_new,
     col_data => \@col_data,
 
@@ -400,6 +409,7 @@ sub merge {
 sub inconsistencies {
   my $self = shift;
 
+  # TODO
   # check if assay is rectangular
   # check if row data is rectangular and has the
 
@@ -423,6 +433,7 @@ sub clone {
   my $self = shift;
   return __PACKAGE__->new(
     assay          => Clone::clone( $self->assay ),
+    na_value       => $self->na_value,
     col_data       => Clone::clone( $self->col_data ),
     col_names      => Clone::clone( $self->col_names ),
     row_names      => Clone::clone( $self->row_names ),
@@ -503,9 +514,17 @@ sub _max_dim {
   return [ scalar(@$aoa), $max_ncol ];
 }
 
-sub _na_fill_2d {
-  my $data = shift;
-  my $dim = shift // [ 0, 0 ];
+sub fill_assay {
+  my $self = shift;
+
+  _Fill_2d($self->assay);
+  return $self;
+}
+
+sub _Fill_2d {
+  my $data     = shift;
+  my $dim      = shift // [ 0, 0 ];
+  my $na_value = shift;
 
   return unless ( $data && ref $data eq 'ARRAY' );
   my $dim_data = _max_dim($data);
@@ -517,22 +536,23 @@ sub _na_fill_2d {
     $data->[$i] //= [];
     next if ( @{ $data->[$i] } == $ncol );
     for ( my $j = 0; $j < $ncol; $j++ ) {
-      $data->[$i][$j] //= $NA_VALUE;
+      $data->[$i][$j] //= $na_value;
     }
   }
   return $data;
 }
 
 sub _na_fill_1d {
-  my $data = shift;
-  my $dim  = shift;
+  my $data     = shift;
+  my $dim      = shift;
+  my $na_value = shift // $NA_VALUE;
 
   return unless ( $data && ref $data eq 'ARRAY' );
   my $len = max( $dim, @$data );
   return unless ($len);
 
   for ( my $i = 0; $i < $len; $i++ ) {
-    $data->[$i] //= $NA_VALUE;
+    $data->[$i] //= $na_value;
   }
   return $data;
 }
@@ -564,6 +584,7 @@ sub aggregate_by_idcs {
 
   return __PACKAGE__->new(
     assay          => \@agg_assay,
+    na_value       => $self->na_value,
     col_names      => $col_names // [],
     row_names      => \@agg_row_names,
     row_data_names => \@agg_row_data_names,
@@ -693,7 +714,7 @@ sub element_apply {
   my $assay = $self->assay;
   for ( my $i = 0; $i < @$assay; $i++ ) {
     my $j = 0;
-    my @row_res = map { $cb->($_, $i, $j++) } @{$assay->[$i] // []};
+    my @row_res = map { $cb->( $_, $i, $j++ ) } @{ $assay->[$i] // [] };
 
     push @res, \@row_res;
   }
@@ -721,13 +742,12 @@ sub apply {
   } elsif ( $dir eq 'c' || $dir == 2 ) {
     return $self->col_apply( $cb, @args );
   } elsif ( $dir eq 'rc' || $dir eq 'cr' || $dir == 3 ) {
-    return $self->element_apply($cb, @args);
+    return $self->element_apply( $cb, @args );
   }
 }
 
 sub slice_by_idcs {
   my ( $self, $idcs ) = @_;
-  #FIXME implement cloning
 
   my @assay_new = map { [ @{$_}[@$idcs] ] } @{ $self->assay };
   my @new_colnames;
@@ -738,6 +758,7 @@ sub slice_by_idcs {
 
   return __PACKAGE__->new(
     assay          => \@assay_new,
+    na_value       => $self->na_value,
     row_names      => Clone::clone( $self->row_names ),
     row_data       => Clone::clone( $self->row_data ),
     col_names      => \@new_colnames,
@@ -786,12 +807,15 @@ sub slice_by_names {
 }
 
 sub each {
-  shift->apply( 1, @_ );
+  my $self = shift;
+  $self->apply( 1, @_ );
+  return $self;
 }
 
 sub uniq {
   my $self = shift;
 
+      no warnings 'uninitialized';
   my %seen;
   return $self->subset(
     sub {
@@ -846,6 +870,12 @@ Return the assay of the summarized experiment.
 
 =head2 meta_data
 
+=head2 na_value
+
+Set the NA value if the object is stored. Internally, while I<in memory>, undef will be
+used. So this value will only have an effect if data is read or written somewhere.
+Default is C<Bio::Gonzales::SummarizedExperiment::NA_VALUE>.
+
 =head1 METHODS
 
 =head2 data
@@ -927,18 +957,81 @@ sensitve and returns a hash in list context and a hash reference in scalar conte
 =head2 json_spew
 =head2 make_consistent
 =head2 merge
+
+Merge two SummarizedExperiment objects.
+
+    use Bio::Gonzales::SummarizedExperiment;
+    my $se_x = Bio::Gonzales::SummarizedExperiment->new(
+      assay => [ [ 1, "homer", "simpson" ], [ 2, "bart", "simpson" ], [ 3, "lisa simpson" ] ],
+      col_names => [qw(user_id first_name surname)]
+    );
+    
+    my $se_y = Bio::Gonzales::SummarizedExperiment->new(
+      assay => [ [ 1, 120 ], [ 2, 20 ] ],
+      col_names => [qw(user_id weight_kg)]
+    );
+    
+    # inner join by default
+    my $merged_se = $se_x->merge($se_y, { by => [ 'user_id' ] });
+    
+    # user_id first_name surname weight_kg
+    # 1       homer      simpson 120
+    # 2       bart       simpson 20
+    # Lisa is missing, because the $se_y lacks weight information.
+
 =head2 names_to_idcs
 =head2 ncol
 =head2 nrow
 =head2 rbind
 =head2 row_apply
+
+Apply a callback to each row. C<$se->row_apply(@args)> is equivalent to C<$se->apply(1, @args)>
+
+
+    use Bio::Gonzales::SummarizedExperiment;
+    my $se = Bio::Gonzales::SummarizedExperiment->new(
+      assay => [ [ 1, "homer", "simpson" ], [ 2, "bart", "simpson" ], [ 3, "lisa", "simpson" ] ],
+      col_names => [qw(user_id first_name surname)]
+    );
+
+    # WITHOUT MODIFYING THE SUMMARIZEDEXPERIMENT OBJECT
+    $se->row_apply(sub { (my $name = $_->[1]) =~ s/[ra]/z/; $name });
+    # [ 'homez', 'bzrt', 'lisz' ];
+
+    $se->extract_col_by_idx(1);
+    # [ 'homer', 'bart', 'lisa' ];
+
+
+    # WITH MODIFYING THE SUMMARIZEDEXPERIMENT OBJECT
+    $se->row_apply(sub { $_->[1] =~ s/[ra]/z/; $_->[1] });
+    # [ 'homez', 'bzrt', 'lisz' ];
+
+    $se->extract_col_by_idx(1);
+    # [ 'homez', 'bzrt', 'lisz' ];
+
+
+
 =head2 row_idx
 =head2 row_idx_map
 =head2 row_idx_match
 =head2 shuffle
+
 =head2 slice_by_idcs
+
+    $se->slice_by_idcs(\@idcs);
+    $se->slice_by_idcs([0,5,13]);
+
+Extract a column-"slice" from the summarized experiment. The indices select the columns.
+
 =head2 slice_by_names
+
 =head2 slurp_assay
+    
+    my $se = Bio::Gonzales::SummarizedExperiment->slurp_assay($source, \%params);
+    my $se = Bio::Gonzales::SummarizedExperiment->slurp_assay("data.csv", { header => 1, sep => ';' });
+
+Create a new summarized experiment from matrix/tabular data.
+
 =head2 sort
 =head2 spew_assay
 =head2 subset
